@@ -5,7 +5,7 @@
  * $Author: kou $
  * $Date: 2008-09-26 13:52:08 $
  *
- * Copyright 2005-2018 Kouhei Sutou <kou@cozmixng.org>
+ * Copyright 2005-2019 Kouhei Sutou <kou@cozmixng.org>
  * Copyright 2005 Øyvind Kolås <pippin@freedesktop.org>
  * Copyright 2004-2005 MenTaLguY <mental@rydia.com>
  *
@@ -122,6 +122,12 @@ cr_destroy (VALUE self)
   return Qnil;
 }
 
+static VALUE
+cr_destroyed (VALUE self)
+{
+  return CBOOL2RVAL (DATA_PTR (self) == NULL);
+}
+
 static VALUE
 cr_destroy_with_destroy_check (VALUE self)
 {
@@ -234,6 +240,12 @@ cr_to_ptr (VALUE self)
                      1, PTR2NUM (_SELF));
 }
 
+static VALUE
+cr_raw_address (VALUE self)
+{
+  return PTR2NUM (_SELF);
+}
+
 
 static VALUE
 cr_restore (VALUE self)
@@ -345,7 +357,7 @@ cr_set_source_rgb (int argc, VALUE *argv, VALUE self)
   if (n == 1 && rb_cairo__is_kind_of (red, rb_cArray))
     {
       VALUE ary = red;
-      n = RARRAY_LEN (ary);
+      n = (int) RARRAY_LEN (ary);
       red = rb_ary_entry (ary, 0);
       green = rb_ary_entry (ary, 1);
       blue = rb_ary_entry (ary, 2);
@@ -382,7 +394,7 @@ cr_set_source_rgba (int argc, VALUE *argv, VALUE self)
   if (n == 1 && rb_cairo__is_kind_of (red, rb_cArray))
     {
       VALUE ary = red;
-      n = RARRAY_LEN (ary);
+      n = (int) RARRAY_LEN (ary);
       red = rb_ary_entry (ary, 0);
       green = rb_ary_entry (ary, 1);
       blue = rb_ary_entry (ary, 2);
@@ -561,7 +573,7 @@ cr_set_dash (int argc, VALUE *argv, VALUE self)
     {
       int i, length;
       double *values;
-      length = RARRAY_LEN (dash_array);
+      length = (int) RARRAY_LEN (dash_array);
       values = ALLOCA_N (double, length);
       if (!values)
         {
@@ -1317,7 +1329,7 @@ cr_show_text_glyphs (VALUE self, VALUE rb_utf8, VALUE rb_glyphs,
 
   cr = _SELF;
   utf8 = RSTRING_PTR (rb_utf8);
-  utf8_len = RSTRING_LEN (rb_utf8);
+  utf8_len = (int) RSTRING_LEN (rb_utf8);
   rb_cairo__glyphs_from_ruby_object (rb_glyphs, &glyphs, &num_glyphs);
   rb_cairo__text_clusters_from_ruby_object (rb_clusters,
                                             &clusters, &num_clusters);
@@ -1717,6 +1729,7 @@ Init_cairo_context (void)
   /* Functions for manipulating state objects */
   rb_define_method (rb_cCairo_Context, "initialize", cr_initialize, 1);
   rb_define_method (rb_cCairo_Context, "destroy", cr_destroy, 0);
+  rb_define_method (rb_cCairo_Context, "destroyed?", cr_destroyed, 0);
   rb_define_method (rb_cCairo_Context, "reference_count",
                     cr_get_reference_count, 0);
 
@@ -1867,6 +1880,8 @@ Init_cairo_context (void)
 
   rb_define_method (rb_cCairo_Context, "to_ptr", cr_to_ptr, 0);
 
+  rb_define_method (rb_cCairo_Context, "raw_address", cr_raw_address, 0);
+
 #if CAIRO_CHECK_VERSION(1, 15, 4)
   /* Logical structure tagging functions */
   {
